// This work is licensed under the Creative Commons Attribution-ShareAlike 4.0
// International Public License.
// Visit https://creativecommons.org/licenses/by-sa/4.0/.
// SPDX-License-Identifier: CC-BY-SA-4.0
//     yuqk ValuePatternModifiers.pov +w1000 +h500 +a0.0 +am2 +r3 +j7 -cc -v -fn +d +p
#version unofficial 3.8; // yuqk
#if (file_exists("version.inc"))
    #include "version.inc"
#end
#if (!defined(Fork_yuqk))
    #error "This POV-Ray SDL code requires the yuqk fork."
#end
global_settings {
    assumed_gamma 1
}
default {
    finish { diffuse 0.0 emission 1.0 }
}
#declare BrownCrayola = srgb <0.68627,0.34902,0.24314>;
background { BrownCrayola }
#declare VarOrthoMult = 6.1/max(image_width/image_height,image_height/image_width);
#declare Camera01z = camera {
    orthographic
    location <0,0,-2>
    direction z
    right VarOrthoMult*x*max(1,image_width/image_height)
    up VarOrthoMult*y*max(1,image_height/image_width)
}
#declare Cyan = srgb <0,1,1>;

// What follows is simple code for plotting what is happing with the scale value
// pattern mechanism and the various available modifications possible with it.
// The general scalar value pattern modifiers are:

// frequency, phase, function_interval, inverse

// and

// ramp_wave, raw_wave, triangle_wave, sine_wave, scallop_wave, cubic_wave
// and poly_wave. Note extremes for values like frequency might not show up well.
#include "functions.inc"
// Function f_planar defined in functions.inc
#declare White = srgb <1,1,1>;
#declare Text00 = text {
    ttf "timrom.ttf" "ValuePatternModifiers"
    0.1, 0
    pigment { color White }
    scale 0.15
    translate <-0.5,-1.4,0>
}
// Wrap gradient <1,0,> in a function{ pattern {}} and start plotting!
#declare Fn_ValuePatModifiers = function {
    pattern { gradient <1,0,0> ramp_wave poly_wave 1 phase 0.0 frequency 1}
}
// This the function used to plot using and isosurface. See Aqua color.
#declare Fn_plot = function (x,y,z) {
    f_planar(y-Fn_ValuePatModifiers(x,y,z),0.01,1)
}
#declare Iso_plot = isosurface {
    function { Fn_plot(x,y,z) }
    contained_by { box { <-3,-3,-0>,<3,3,0.1> } }
    threshold 0
    accuracy 0.0005
    max_gradient 1.1
    pigment { color Cyan }
}
#declare Red = srgb <1,0,0>;
#declare CylinderX = cylinder {
    -1*x, 1*x, 0.01
    pigment { color Red }
}
#declare Green = srgb <0,1,0>;
#declare CylinderY = cylinder {
    -1*y, 1*y, 0.01
    pigment { color Green }
}
#declare Blue = srgb <0,0,1>;
#declare CylinderZ = cylinder {
    -1*z, 1*z, 0.01
    pigment { color Blue }
}
#declare Yellow = srgb <1,1,0>;
#declare CylinderZY = cylinder {
    -1*z, 1*z, 0.01
    pigment { color Yellow }
}
#declare Obj00 = object {
    CylinderZ
    rotate x*90
    scale <1,0.5,1>
    translate <-0.5,0,0>
}
#declare Obj01 = object {
    CylinderZ
    rotate x*90
    scale <1,0.5,1>
    translate <+0.5,0,0>
}
#declare Obj02 = object {
    CylinderZ
    rotate x*90
    scale <1,0.5,1>
    translate <-1.5,0,0>
}
#declare Obj03 = object {
    CylinderZ
    rotate x*90
    scale <1,0.5,1>
    translate <+1.5,0,0>
}
#declare Obj04 = object {
    CylinderZ
    rotate x*90
    scale <1,0.5,1>
    translate <-2.5,0,0>
}
#declare Obj05 = object {
    CylinderZ
    rotate x*90
    scale <1,0.5,1>
    translate <+2.5,0,0>
}
#declare Obj06 = object {
    CylinderZY
    rotate x*90
    translate <-1.0,0,0>
}
#declare Obj07 = object {
    CylinderZY
    rotate x*90
    translate <+1.0,0,0>
}
#declare Obj08 = object {
    CylinderZY
    rotate x*90
    translate <-2.0,0,0>
}
#declare Obj09 = object {
    CylinderZY
    rotate x*90
    translate <+2.0,0,0>
}
#declare Obj10 = object {
    CylinderZY
    rotate x*90
    translate <-3.0,0,0>
}
#declare Obj11 = object {
    CylinderZY
    rotate x*90
    translate <+3.0,0,0>
}
#declare U00 = union {
    object { Obj00 }
    object { Obj01 }
    object { Obj02 }
    object { Obj03 }
    object { Obj04 }
    object { Obj05 }
    object { Obj06 }
    object { Obj07 }
    object { Obj08 }
    object { Obj09 }
    object { Obj10 }
    object { Obj11 }
    object { CylinderX }
    object { CylinderY }
    object { CylinderZ }
}

//--- scene ---
    camera { Camera01z }
    object { Text00 }
    object { Iso_plot }
    object { U00 }
