" Vim syntax file
" Language: POV-Ray v3.8 (yuqk fork) Initialization (.ini) files
" Maintainer: William F. Pokorny
" URLs:
"    http://news.povray.org/povray.binaries.programming/
"    http://news.povray.org/povray.pov4.discussion.general/
" Copyright:
"    Copyright (c) 1991-2024 Persistence of Vision Raytracer Pty. Ltd.
" License:
"    This work is licensed under the Creative Commons Attribution-ShareAlike 4.0
"    International Public License.
"    Visit https://creativecommons.org/licenses/by-sa/4.0/
" Last Change: Sat Jan 20 02:35:40 -0500 2024

if exists("b:current_syntax")
    finish
endif

" TODO. Why do we see the echo below twice when uncommented?
" echom "Debugging. Reading yuqkini.vim."

let s:cpo_save = &cpo
set cpo&vim

" POV-Ray ini files and command line options are case agnostic
syn case ignore

" Comments. POV-Ray ini files support ';' and '#' anywhere in line
syn match yuqkiniComment "[#;].*$" contains=yuqkTodo
syn keyword yuqkiniTodo TODO FIXME NB NOTE contained
syn match yuqkCommentiniErrors "[a-z,A-Z,0-9_\.][#;]\+\ze.*$"

" String literals. Used with file names having spaces (not recommended).
" Note. The yuqk command line forms require traditional linux/unix double quote escapes.
" Official POV-Ray releases require odd escape sequences. Ref: Github issue 448.
syn region  yuqkiniString        start=+"+ skip=+\\\\\|\\"+ end=+"+ contains=yuqkiniStringSpecial
syn match   yuqkiniStringSpecial "\\x\x\+\|\\u\x\{1,6\}" contained containedin=yuqkiniString
syn match   yuqkiniStringErrors  "\\u\x\{7,\}" contained containedin=yuqkiniString

" Active command line / ini file options
syn keyword yuqkiniKeywords all_console all_file antialias antialias_confidence antialias_depth antialias_gamma
syn keyword yuqkiniKeywords antialias_min_depth antialias_threshold append_file bits_per_color bits_per_colour
syn keyword yuqkiniKeywords bounding bounding_method bounding_threshold bsp_baseaccesscost bsp_childaccesscost
syn keyword yuqkiniKeywords bsp_isectcost bsp_maxdepth bsp_misschance clock clockless_animation compression continue_trace
syn keyword yuqkiniKeywords create_continue_trace_log create_ini cyclic_animation debug_console debug_file declare
syn keyword yuqkiniKeywords display display_gamma dither dither_method end_column end_row fatal_console fatal_error_command
syn keyword yuqkiniKeywords fatal_error_return fatal_file file_gamma final_clock final_frame frame_step grayscale_output
syn keyword yuqkiniKeywords greyscale_output height high_reproducibility include_header include_ini initial_clock
syn keyword yuqkiniKeywords initial_frame input_file_name jitter jitter_amount library_path max_image_buffer_memory
syn keyword yuqkiniKeywords output_alpha output_file_name output_file_type output_to_file pause_when_done post_frame_command
syn keyword yuqkiniKeywords post_frame_return post_scene_command post_scene_return pre_frame_command pre_frame_return
syn keyword yuqkiniKeywords pre_scene_command pre_scene_return preview_end_size preview_start_size quality radiosity_file_name
syn keyword yuqkiniKeywords radiosity_from_file radiosity_to_file radiosity_vain_pretrace real_time_capture real_time_raytracing
syn keyword yuqkiniKeywords remove_bounds render_block_size render_block_step render_console render_file render_pattern
syn keyword yuqkiniKeywords sampling_method split_unions start_column start_row statistic_console statistic_file
syn keyword yuqkiniKeywords stochastic_seed subset_end_frame subset_start_frame user_abort_command user_abort_return
syn keyword yuqkiniKeywords verbose version warning_console warning_file warning_level width work_threads

" Command line / ini file options removed from yuqk
syn keyword yuqkiniKeywordErrors buffer_output buffer_size create_histogram draw_vistas field_render histogram_grid_size
syn keyword yuqkiniKeywordErrors histogram_name histogram_type light_buffer odd_field palette radiosity test_abort
syn keyword yuqkiniKeywordErrors test_abort_count video_mode vista_buffer

" Active command line / ini file flags
" ====================================
" Flag form of antialias(a) and antialias_threshold(value)
syn match yuqkiniFlags "[-+]a\(\s\+\|$\)"
syn match yuqkiniFlags "[-+]a[0-9.]\+\(\s\+\|$\)"
" Flag form of antialias_confidence
syn match yuqkiniFlags "[-+]ac0\.[0-9]\+\(\s\+\|$\)"
" Flag form of antialias_gamma
syn match yuqkiniFlags "[-+]ag[0-9.]\+\(\s\+\|$\)"
" Flag form of sampling_method
syn match yuqkiniFlags "[-+]am[1-3]\{1\}\(\s\+\|$\)"

" Flag form of bounding
syn match yuqkiniFlags "[-+]b\(\s\+\|$\)"
" Flag form of bounding_method(1 or 2)
syn match yuqkiniFlags "[-+]bm[1-2]\{1\}\(\s\+\|$\)"

" Flag form of render_block_size
syn match yuqkiniFlags "[-+]bs\([4-9]\{1\}\|[1-9][0-9]\+\)\(\s\+\|$\)"

" Flag form of continue_trace. Only works if +cc created state file in prior partial render 
syn match yuqkiniFlags "[-+]c\(\s\+\|$\)"
" Flag form of create_continue_trace_log. Default is on. 
" Note. Slows render and files can be large. Use -cc, if confidence in render completion high.
syn match yuqkiniFlags "[-+]cc\(\s\+\|$\)"

" Flag form of display. Only works if preview display option(s) compiled into POV-Ray.
syn match yuqkiniFlags "[-+]d\(\s\+\|$\)"

" Flag form of preview_end_size
syn match yuqkiniFlags "[-+]ep[1-9]\{1\}[0-9]*\(\s\+\|$\)"

" Flag form of subset_end_frame
syn match yuqkiniFlags "[-+]ef[0-9]\+[\.]*[0-9]*\(\s\+\|$\)"

" Flag form of output_file_type
syn match yuqkiniFlags "[-+]\(f\|f[b-c,e,h,j,n,p,s-t]\{1\}g\{,1\}\([1-9]\{,1\}\|1[0-6]\{,1\}\)\)\(\s\+\|$\)"

" Flag form of end_column. Pixel count or percentage.
syn match yuqkiniFlags "[-+]\(ec100\|ec[0-9]\+[\.]*[0-9]*\)\(\s\+\|$\)"
" Flag form of end_row. Pixel count or percentage.
syn match yuqkiniFlags "[-+]\(er100\|er[0-9]\+[\.]*[0-9]*\)\(\s\+\|$\)"

" Flag form of Create_Ini=file
syn match yuqkiniFlags "[-+]gi\(\s\+\|$\)"
syn match yuqkiniFlags "[-+]gi\ze\""
syn match yuqkiniFlags "[-+]gi\S\+\(\s\+\|$\)"

" Output stream control
" ---------------------
" Note! Since the introduction of threading it's been the case some output
" is written directly to stdout/cout and stderr/cerr; often from the render threads.
" The status partly code work never completed. However, it is also due re-arranging 
" code with the aim of making the differing parts independent. This bit is work 
" toward callable libraries - where the defined console streams make less sense. 
" ---
" Flag form of debug_console=on/off and debug_file=file
syn match yuqkiniFlags "\([-+]gd\ze\"\)\|\([-+]gd\S\+\(\s\+\|$\)\)\|\([-+]gd\(\s\+\|$\)\)"
syn match yuqkiniFlagErrors "\([-+]gd\s\+\ze\"\)\|\([-+]gd\s\+\ze\([^-+]\+\|$\)\)"
" Flag form of fatal_console=on/off and fatal_file=file
syn match yuqkiniFlags "\([-+]gf\ze\"\)\|\([-+]gf\S\+\(\s\+\|$\)\)\|\([-+]gf\(\s\+\|$\)\)"
syn match yuqkiniFlagErrors "\([-+]gf\s\+\ze\"\)\|\([-+]gf\s\+\ze\([^-+]\+\|$\)\)"
" Flag form of render_console=on/off and render_file=file
syn match yuqkiniFlags "\([-+]gr\ze\"\)\|\([-+]gr\S\+\(\s\+\|$\)\)\|\([-+]gr\(\s\+\|$\)\)"
syn match yuqkiniFlagErrors "\([-+]gr\s\+\ze\"\)\|\([-+]gr\s\+\ze\([^-+]\+\|$\)\)"
" Flag form of statistic_console=on/off and statistic_file=file
syn match yuqkiniFlags "\([-+]gs\ze\"\)\|\([-+]gs\S\+\(\s\+\|$\)\)\|\([-+]gs\(\s\+\|$\)\)"
syn match yuqkiniFlagErrors "\([-+]gs\s\+\ze\"\)\|\([-+]gs\s\+\ze\([^-+]\+\|$\)\)"
" Flag form of warning_console=on/off and warning_file=file
syn match yuqkiniFlags "\([-+]gw\ze\"\)\|\([-+]gw\S\+\(\s\+\|$\)\)\|\([-+]gw\(\s\+\|$\)\)"
syn match yuqkiniFlagErrors "\([-+]gw\s\+\ze\"\)\|\([-+]gw\s\+\ze\([^-+]\+\|$\)\)"
" Flag form of all_console=on/off and all_file=file
syn match yuqkiniFlags "\([-+]ga\ze\"\)\|\([-+]ga\S\+\(\s\+\|$\)\)\|\([-+]ga\(\s\+\|$\)\)"
syn match yuqkiniFlagErrors "\([-+]ga\s\+\ze\"\)\|\([-+]ga\s\+\ze\([^-+]\+\|$\)\)"
" Flag form of append_file boolean for all output streams.
syn match yuqkiniFlags "[-+]gp\(\s\+\|$\)"

" Flag form of height
syn match yuqkiniFlags "[-+]h[1-9]\{1\}[0-9]*\(\s\+\|$\)"

" Flag form of include_header
syn match yuqkiniFlagErrors "[-+]hi\(\s\+\|$\)"
syn match yuqkiniFlags "[-+]hi\ze\""
syn match yuqkiniFlags "[-+]hi\S\+\(\s\+\|$\)"

" Flag form of high_reproducibility
" Note. Works to significant exent, but mostly not as well rendering with a single thread.
syn match yuqkiniFlags "[-+]hr\(\s\+\|$\)"

" Flag form of input_file_name
syn match yuqkiniFlags "\([-+]i\ze\"\)\|\([-+]i\S\+\(\s\+\|$\)\)"
syn match yuqkiniFlagErrors "\([-+]i\s\+\ze\"\)\|\([-+]i\s\+\ze\)\|\([-+]i\ze$\)"

" Flag form of jitter(j) and optional jitter_ammount(value)
syn match yuqkiniFlags "[-+]\(j\|j[0-9]\+[\.]*[0-9]*\)\(\s\+\|$\)"

" Flag form of clock
syn match yuqkiniFlags "[-+]k[0-9]\+[\.]*[0-9]*\(\s\+\|$\)"
" Flag form of initial_frame
" Note! To render subsets of frames use subset frame options
syn match yuqkiniFlags "[-+]kfi[0-9]\+\(\s\+\|$\)"
" Flag form of final_frame
syn match yuqkiniFlags "[-+]kff[0-9]\+\(\s\+\|$\)"
" Flag form of initial_clock
syn match yuqkiniFlags "[-+]ki[0-9]\+[\.]*[0-9]*\(\s\+\|$\)"
" Flag form of final_clock
syn match yuqkiniFlags "[-+]kf[0-9]\+[\.]*[0-9]*\(\s\+\|$\)"

" Flag form of cyclic_animation. Note. In yuqk this used with normal animation
" renders and the clockless/real time raytracing animations. Default is -kc
syn match yuqkiniFlags "[-+]kc\(\s\+\|$\)"
" Flag form of clockless_animation. Note. Used today only with +rtr.
syn match yuqkiniFlags "[-+]kla\(\s\+\|$\)"

" Flag form of library_path
" Note. Multiples of this flag may be used to include additional directories.
syn match yuqkiniFlags "\([-+]l\ze\"\)\|\([-+]l\S\+\(\s\+\|$\)\)"
syn match yuqkiniFlagErrors "\([-+]l\s\+\ze\"\)\|\([-+]l\s\+\ze\)\|\([-+]l\ze$\)"

" Flag form of bounding_threshold(integer value)
syn match yuqkiniFlags "[-+]mb[0-9]\+\(\s\+\|$\)"

" Flag form of max_image_buffer_memory
" Note. IIRC this setting a concern more for POV-Ray running on windows with file backing.
syn match yuqkiniFlags "[-+]mi[1-9]\{1\}[0-9]*\(\s\+\|$\)"

" Flag form of version
syn match yuqkiniFlags "[-+]mv\([3]\{1\}\.[5-8]\|[4]\{1\}\.[0-1]\)\(\s\+\|$\)"

" Flag form of output_file_name
syn match yuqkiniFlags "\([-+]o\ze\"\)\|\([-+]o\S\+\(\s\+\|$\)\)"
syn match yuqkiniFlagErrors "\([-+]o\s\+\ze\"\)\|\([-+]o\s\+\ze\)\|\([-+]o\ze$\)"

" Flag form of pause_when_done.
syn match yuqkiniFlags "[-+]p\(\s\+\|$\)"

" Flag form of quality
" Note. Levels 10 and 11 are not currently used. So, 9 the highest.
syn match yuqkiniFlags "[-+]q\([0-9]\{1\}\|10\|11\)\(\s\+\|$\)"

" Flag form of antialias_depth
syn match yuqkiniFlags "[-+]r[1-9]\+\(\s\+\|$\)"
" Flag form of antialias_min_depth
syn match yuqkiniFlags "[-+]rm[0-9]\+\(\s\+\|$\)"

" Flag form of render_pattern
syn match yuqkiniFlags "[-+]rp\([0-5]\{1\}\)\(\s\+\|$\)"
" Flag form of render_block_step
syn match yuqkiniFlags "[-+]rs\([1-9]\{1\}[0-9]*\)\(\s\+\|$\)"

" Flag form of Radiosity_File
syn match yuqkiniFlags "\([-+]rf\ze\"\)\|\([-+]rf\S\+\(\s\+\|$\)\)"
syn match yuqkiniFlagErrors "\([-+]rf\s\+\ze\"\)\|\([-+]rf\s\+\ze\)\|\([-+]rf\ze$\)"
" Flag form of Radiosity_From_File
syn match yuqkiniFlags "[-+]rfi\(\s\+\|$\)"
" Flag form of Radiosity_To_File
syn match yuqkiniFlags "[-+]rfo\(\s\+\|$\)"
" Flag form of radiosity_vain_pretrace
syn match yuqkiniFlags "[-+]rvp\(\s\+\|$\)"

" --- real time raytracing / image capture 
" Flag form of real_time_capture. Note. This is a yuqk only option which today
" writes to numbered, color pfm files. The image file creation also bypasses the
" usual povmsg data to image buffer date passing which often backs up causing huge
" storage bubbles.

" The color pfm output is the only true 32bit/channel POV-Ray image output. The
" hdr and exr (currently only half float) are good for dynamic range only. There
" is currently no POV-Ray (yuqk) pfm input capability. Outside utilities must convert
" to other more mainstream formats for ffmpeg or POV-Ray use as input.

" Note. Single image renders to pfm output can be accomplished by using:
" +rtr +rtc -kla. The key being -kla. Only the first camera of one or more
" cameras will be rendered and written to a pfm file.
syn match yuqkiniFlags "[-+]rtc\(\s\+\|$\)"
" Flag form of real_time_raytracing. This works only with the clockless, camera 
" list kind of animations. In other words +kla and +rtr must be used together. 
syn match yuqkiniFlags "[-+]rtr\(\s\+\|$\)"
" ---

" Flag form of start_column. Pixel count or percentage.
syn match yuqkiniFlags "[-+]\(sc0\|sc[0-9]\+[\.]*[0-9]*\)\(\s\+\|$\)"
" Flag form of start_row. Pixel count or percentage.
syn match yuqkiniFlags "[-+]\(sr0\|sr[0-9]\+[\.]*[0-9]*\)\(\s\+\|$\)"

" Flag form of subset_start_frame
syn match yuqkiniFlags "[-+]sf[0-9]\+[\.]*[0-9]*\(\s\+\|$\)"

" Flag form of preview_start_size
syn match yuqkiniFlags "[-+]sp[1-9]\{1\}[0-9]*\(\s\+\|$\)"

" Flag form of frame_step
syn match yuqkiniFlags "[-+]stp[0-9]\+\(\s\+\|$\)"

" Flag form of split_unions (default is off)
" Note. Manual bounding can set photon shooting target to entire union.
syn match yuqkiniFlags "[-+]su\(\s\+\|$\)"
" Flag form of stochastic_seed(value)
syn match yuqkiniFlags "[-+]ss[0-9]\+\(\s\+\|$\)"

" Flag form of dither and dither_mode
syn match yuqkiniFlags "\([-+]th\(\s\+\|$\)\)\|\([-+]th\(b2\|b3\|b4\|bn\|bnx\|at\|bk\|d1\|d2\|fs\|jn\|s2\|s3\|st\)\(\s\+\|$\)\)"

" Flag form of remove_bounds (default is off)
" Note. Must also have background / env transparency in scene - otherwise pointless
syn match yuqkiniFlags "[-+]ua\(\s\+\|$\)"

" Flag form of remove_bounds (default is on)
" Note. There are exceptions where bounding is not removed
" Plus. There are cases where users should NOT remove bounds due poor or no auto-bounding
syn match yuqkiniFlags "[-+]ur\(\s\+\|$\)"

" Flag form of verbose. Default is on. Most noticeable effect is text progress messages.
syn match yuqkiniFlags "[-+]v\(\s\+\|$\)"

" Flag form of width
syn match yuqkiniFlags "[-+]w\([1-9]\{1\}[0-9]*\)\(\s\+\|$\)"
" Flag form of warning_level. Note. Support for this feature has eroded over time.
syn match yuqkiniFlags "[-+]wl[0-9]\{1,2\}\(\s\+\|$\)"
" Flag form of work_threads
syn match yuqkiniFlags "[-+]wt[0-9]\+\(\s\+\|$\)"

" Active ini file options without flag equivalents.
" ---
" (You can add long ini forms to the command line)
" ====================================
" compression=<target_ratio> for jpeg output
" declare=Var00=0.123
" display_gamma=(n.n|bt709|bt2020|srgb)
" file_gamma=(n.n|bt709|bt2020|srgb)
" include_ini=<filename> // Note. This explicit form, safer than guessed raw filename.
" --- Shell command options
" pre_scene_command=s
" pre_frame_command=s
" post_scene_command=s
" post_frame_command=s
" user_abort_command=s
" fatal_abort_command=s
" -
" where the string substitutions are:
" %o %s %n %k %h %w %%
" --- Shell command return actions
" pre_scene_return=s
" pre_frame_return=s
" post_scene_return=s
" post_frame_return=s
" user_abort_return=s
" fatal_abort_return=s
" -
" where the string substitutions are:
" i s a q u f 
" ---
" (The animation command optios only apply to full animation and not RTR.)
" ---

" Command line / ini file flags removed from yuqk and deprecated in the official v3.8
" ====================================
" Flag form of video_mode or pallete.
" Note. Video color mode and pallete no longer needed with modern displays.
" POV-Ray allows bogus settings in v3.7 on, yuqk doesn't. 
syn match yuqkiniFlagErrors "[-+]d\S\+\(\s\+\|$\)"

" Flag form of histogram_file
syn match yuqkiniFlagErrors "\([-+]hn[^\"]\ze\)\|\([-+]hn\s*\ze\"\)\|\([-+]hn\)\(\s\+\|$\)"
" Flag form of histogram_grid_size
syn match yuqkiniFlagErrors "\([-+]hs\(\s\+\|$\)\)\|\([-+]hs\d\+\.\d\+\(\s\+\|$\)\)"
" Flag form of histogram_type
syn match yuqkiniFlagErrors "[-+]ht\(\s\+\|$\)\|[-+]ht\(c\|n\|p\|s\|t\|x\)\(\s\+\|$\)"

" Flag form of radiosity. This long triggered with SDL radiosity block instead.
syn match yuqkiniFlagErrors "[-+]qr\(\s\+\|$\)"

" Flag form of draw_vistas
syn match yuqkiniFlagErrors "[-+]ud\(\s\+\|$\)"
" Flag form of light_buffer
syn match yuqkiniFlagErrors "[-+]ul\(\s\+\|$\)"
" Flag form of vista_buffer
syn match yuqkiniFlagErrors "[-+]uv\(\s\+\|$\)"

" Flag form of field_render=bool
syn match yuqkiniFlagErrors "[-+]uf\(\s\+\|$\)"
" Flag form of odd_field=bool
syn match yuqkiniFlagErrors "[-+]uo\(\s\+\|$\)"

" Flag form of test_abort=on|off and test_abort_count <positive integer>
" Note. The regex can precent leadng first 0, but current regex allows it
syn match yuqkiniFlagErrors "[-+]\(x\|x[0-9]\{1\}[0-9]*\)\(\s\+\|$\)"

" POV-Ray (yuqk form) Unix / linux command line flags
" ===================================================
" Note! These are NOT ini file options and only work from the command line
" ===================================================
" Preview display options. This overrides +-d.
syn match yuqkUnixFlags "[-]\{2\}preview\s\+x11\(\s\+\|$\)"
syn match yuqkUnixFlags "[-]\{2\}preview\s\+sdl\(\s\+\|$\)"
syn match yuqkUnixFlags "[-]\{2\}preview\s\+text\(\s\+\|$\)"
syn match yuqkUnixFlags "[-]\{1\}y\s\+x11\(\s\+\|$\)"
syn match yuqkUnixFlags "[-]\{1\}y\s\+sdl\(\s\+\|$\)"
syn match yuqkUnixFlags "[-]\{1\}y\s\+text\(\s\+\|$\)"

" General options
syn match yuqkUnixFlags "[-]\{1,2\}authors\(\s\+\|$\)"
syn match yuqkUnixFlags "[-]\{2\}generation\(\s\+\|$\)"
syn match yuqkUnixFlags "[-]\{1,2\}help\(\s\+\|$\)"
syn match yuqkUnixFlags "[-]h\(\s\+\|$\)"
syn match yuqkUnixFlags "[-][?]\(\s\+\|$\)"
syn match yuqkUnixFlags "[-]\{1,2\}libraries\(\s\+\|$\)"
syn match yuqkUnixFlags "[-]\{1,2\}license\(\s\+\|$\)"
" Linux/Unix --version and -version collide with ini keyword so not highlighted
" as a yuqk Unix flag.

" POV-Ray (yuqk form) Unix / linux command line flags which are errors
" ===================================================

" The --benchmark|-benchmark flag. The yuqk fork removed this feature.
syn match yuqkUnixFlagErrors "[-]\{1,2\}benchmark\(\s\+\|$\)"

" The -generation flavor of flag not supported?. Only '-' form not.
syn match yuqkUnixFlagErrors "\(^\|\s+\)[-]\{1\}generation\(\s\+\|$\)"

" Command line / ini file keywords also POV-Ray v3.8 / yuqk v3.8 keywords 
" ====================================
syn keyword yuqkiniKeywordArgs false no off on true yes bt2020 bt709 srgb
" Links to highlights. Mostly not aligning to predefined highlight categories.
" With POV-Ray, yuqk included, keywords often fit multiple categories. Herein 
" only aiming for generalized highlighting buckets. 
hi def link yuqkiniComment          Comment
hi def link yuqkiniTodo             Todo
hi def link yuqkiniKeywords         Type
hi def link yuqkiniFlags            Identifier
hi def link yuqkiniKeywordArgs      PreProc
hi def link yuqkiniString           Constant
hi def link yuqkUnixFlags           Statement

hi def link yuqkCommentiniErrors    Error
hi def link yuqkiniKeywordErrors    Error
hi def link yuqkiniFlagErrors       Error
hi def link yuqkUnixFlagErrors      Error
hi def link yuqkiniStringErrors     Error

" For smallish ini files expect performance of fromstart OK.
syntax sync fromstart

let b:current_syntax = "yuqkini"

let &cpo = s:cpo_save
unlet s:cpo_save

" SPDX-License-Identifier: CC-BY-SA-4.0
